#include "SimpleLed.h"

SimpleLed::SimpleLed(int pinRed, int pinGreen, int pinBlue) {
  this->pin_red = pinRed;
  this->pin_green = pinGreen;
  this->pin_blue = pinBlue;
}

void SimpleLed::begin()
{
  pinMode(this->pin_red, OUTPUT);
  pinMode(this->pin_green, OUTPUT);
  pinMode(this->pin_blue, OUTPUT);

  this->setColor(color);
}

void SimpleLed::setColor(const Color &color)
{
  this->color.red = color.red;
  this->color.green = color.green;
  this->color.blue = color.blue;

  analogWrite(this->pin_red, 255 * color.red);
  analogWrite(this->pin_green, 255 * color.green);
  analogWrite(this->pin_blue, 255 * color.blue);
}

void SimpleLed::translateToColor(const Color &color, int duration_millis)
{
  const Color colorStart = {
    this->color.red,
    this->color.green,
    this->color.blue
  };

  const Color colorDelta = {
    color.red - this->color.red,
    color.green - this->color.green,
    color.blue - this->color.blue
  };

  long t_start = millis();
  long t_end = t_start + duration_millis;
    
  while (millis() < t_end) {
    float progress = (float)(millis() - t_start) / (float)duration_millis;

    const Color colorCurrent = {
      colorStart.red + colorDelta.red * progress,
      colorStart.green + colorDelta.green * progress,
      colorStart.blue + colorDelta.blue * progress
    };

    setColor(colorCurrent);
  }

  setColor(color);
}

